#include <stm32f0xx.h>
#include <callback.hpp>
using namespace stm32f072xb;

static volatile TimerInterruptCallback* currentCallback;

extern "C" void TIM6_DAC_IRQHandler() {
	if (currentCallback != nullptr) {
		((TimerInterruptCallback*)currentCallback)->handle();
	}
	CLEAR_BIT(TIM6->SR, TIM_SR_UIF);  // TIM6 status register :: Clear interrupt (p. 557)
}

TimerInterruptCallback::TimerInterruptCallback() : _timer(setupBasicTimer()) {
	_callback = (vTimerInterruptCallback)[](void*){};
	_currentIteration = 0;
	_maxIteration = 0;
	_sleepIteration = 0;
}

TimerInterruptCallback::TimerInterruptCallback(vTimerInterruptCallback callback, int repeat, int delay)
    : _timer(setupBasicTimer()) {
	_callback = callback;
	_currentIteration = 0;
	_maxIteration = repeat;
	_sleepIteration = delay;

	currentCallback = this;
}

void TimerInterruptCallback::start(const uint16_t autoReloadValue, const uint16_t prescalerValue) {
	_currentIteration = 0;
	_timer.start(autoReloadValue, prescalerValue);
}

void TimerInterruptCallback::handle() {
	_currentIteration++;
	if ((_maxIteration == 0) || (_currentIteration <= _maxIteration)) {
		_callback((void*)0);
	} else {
		_timer.stop();
	}
}

BasicTimer TimerInterruptCallback::setupBasicTimer() {
	constexpr uint32_t interruptPriority = 0;  // 0 = highest priority

	BasicTimerInit init{TIM6,
	                    Counter::DISABLE,
	                    UevUpdate::ENABLE,
	                    UpdateRequestSource::DISABLE,
	                    OnePulse::DISABLE,
	                    AutoReload::ENABLE,
	                    MasterModeSelection::RESET,
	                    interruptPriority};

	BasicTimer timer6{init};

	return timer6;
}
